package net.shadowfacts.cacao.view.button

import net.shadowfacts.cacao.CacaoScreen
import net.shadowfacts.cacao.Window
import net.shadowfacts.cacao.geometry.Point
import net.shadowfacts.cacao.geometry.Rect
import net.shadowfacts.cacao.util.MouseButton
import net.shadowfacts.cacao.view.View
import net.shadowfacts.cacao.viewcontroller.ViewController
import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import java.util.concurrent.CompletableFuture

/**
 * @author shadowfacts
 */
class ToggleButtonTests {

	companion object {
		@BeforeAll
		@JvmStatic
		fun setupAll() {
			System.setProperty("cacao.drawing.disabled", "true")
		}
	}

	lateinit var screen: CacaoScreen
	lateinit var window: Window
	lateinit var viewController: ViewController

	val view: View
		get() = viewController.view

	@BeforeEach
	fun setup() {
		screen = CacaoScreen()
		viewController = object: ViewController() {
			override fun loadView() {
				view = View(Rect(0.0, 0.0, 1000.0, 1000.0))
			}
		}
		window = screen.addWindow(Window(viewController))
	}

	@Test
	fun testHandlerCalled() {
		val called = CompletableFuture<Boolean>()
		val button = view.addSubview(ToggleButton(false).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = bounds
			handler = {
				called.complete(true)
			}
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertTrue(called.getNow(false))
	}

	@Test
	fun testTogglesValues() {
		val button = view.addSubview(ToggleButton(false).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = bounds
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertTrue(button.state)
		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.LEFT))
		assertFalse(button.state)
	}

	@Test
	fun testMiddleClickDoesNotChangeValue() {
		val button = view.addSubview(ToggleButton(false).apply {
			frame = Rect(0.0, 0.0, 25.0, 25.0)
			content.frame = bounds
		})

		assertTrue(window.mouseClicked(Point(5.0, 5.0), MouseButton.MIDDLE))
		assertFalse(button.state)
	}

}